require 'common/plugin'
require 'common/user'

# Code that all Agendav plugins ({AgendavPrune}, {AgendavRm},
# {AgendavMv}) share.
module AgendavPlugin

  # We implement the Plugin "interface."
  include Plugin


  # Initialize this Agendav {Plugin} with values in *cfg*.
  #
  # @param cfg [Configuration] the configuration for this plugin.
  #
  def initialize(cfg)
    @db_hash = {
      :host     => cfg.agendav_dbhost,
      :port     => cfg.agendav_dbport,
      :options  => cfg.agendav_dbopts,
      :tty      => cfg.agendav_dbtty,
      :dbname   => cfg.agendav_dbname,
      :user     => cfg.agendav_dbuser,
      :password => cfg.agendav_dbpass }
  end


  # Return a list of Agendav users.
  #
  # @return [Array<User>] a list of users contained in the
  #   Agendav database.
  #
  def list_users()
    users = []

    connection = PG::Connection.new(@db_hash)

    sql_query  = '(SELECT username FROM prefs)'
    sql_query += 'UNION'
    sql_query += '(SELECT user_from FROM shared);'

    begin
      connection.query(sql_query) do |result|
        users = result.field_values('username')
      end
    ensure
      # Make sure the connection gets closed even if the query explodes.
      connection.close()
    end

    return users.map{ |u| User.new(u) }
  end

end
