require 'pg'

require 'common/agendav_plugin'
require 'mv/mv_plugin'


# Handle moving (renaming) Agendav users in its database. Agendav has
# no concept of domains.
#
class AgendavMv

  include AgendavPlugin
  include MvPlugin

  # Move the user *src* to *dst* within the Agendav database. This
  # should "rename" him in _every_ table where he is referenced.
  #
  # This can fail is *src* does not exist, or if *dst* already exists
  # before the move. It should also be an error if the destination
  # domain doesn't exist. But Agendav doesn't know about domains, so
  # we let that slide.
  #
  # @param src [User] the source user to be moved.
  #
  # @param dst [User] the destination user being moved to.
  #
  def mv_user(src, dst)
    raise NonexistentUserError.new(src.to_s()) if not user_exists(src)
    raise UserAlreadyExistsError.new(dst.to_s()) if user_exists(dst)

    sql_queries = ['UPDATE prefs SET username = $1 WHERE username = $2;']
    sql_queries << 'UPDATE shared SET user_from = $1 WHERE user_from = $2;'
    sql_queries << 'UPDATE shared SET user_which = $1 WHERE user_which = $2;'

    connection = PG::Connection.new(@db_hash)
    begin
      sql_queries.each do |sql_query|
        connection.query(sql_query, [dst.to_s(), src.to_s()])
      end
    ensure
      # Make sure the connection gets closed even if a query explodes.
      connection.close()
    end
  end

end
