require 'pg'

require 'common/agendav_plugin'
require 'rm/rm_plugin'


# Handle the removal of Agendav users from its database. Agendav has
# no concept of domains.
#
class AgendavRm

  include AgendavPlugin
  include RmPlugin


  # Remove *user* from the Agendav database. This should remove him
  # from _every_ table in which he is referenced.
  #
  # @param user [User] the user to remove.
  #
  def remove_user(user)
    raise NonexistentUserError.new(user.to_s()) if not user_exists(user)

    sql_queries = ['DELETE FROM prefs WHERE username = $1;']
    sql_queries << 'DELETE FROM shared WHERE user_from = $1;'

    connection = PG::Connection.new(@db_hash)
    begin
      sql_queries.each do |sql_query|
        connection.query(sql_query, [user.to_s()])
      end
    ensure
      # Make sure the connection gets closed even if a query explodes.
      connection.close()
    end
  end

end
